@@ -61,8 +61,7 @@ class SmartDriver(SeleniumDriverCore):
                     "b64": screenshotBase64,
                     "use_gclf":  True
                 }
-                # r = requests.post(url='http://127.0.0.1:5002/classify', json=data, timeout=100)
-                r = requests.post(url='http://34.102.26.22:5002/classify', json=data, timeout=100)
+                r = requests.post(url='http://kirfuzz.dev-tools.ai:5002/classify', json=data, timeout=100)
                 raw_content = r.content
                 elems = json.loads(raw_content)
                 for e in elems:
@@ -230,19 +229,13 @@ class SmartDriver(SeleniumDriverCore):
                         'api_key': self.api_key,
                         'label': element_name,
                         'test_case_name': self.test_case_uuid,
+                        'do_exact_match_first': self.do_exact_match_first,
                         }
-                classify_url = self.url + '/detect'
                 start = time.time()
-                for _ in range(2):
-                    try:
-                        r = requests.post(classify_url, json=data, verify=False, timeout=10)
-                        break
-                    except Exception:
-                        log.error('error during detect')
+                response = self.make_json_post_request('/detect', data, self.detect_timeout_message, self.detect_timeout)
                 end = time.time()
                 if self.debug:
                     print(f'Classify time: {end - start}')
-                response = r.json()
                 if not response['success']:
                     # Could be a failure or an element off screen
                     response = self._classify_full_screen(element_name)
@@ -291,8 +284,7 @@ class SmartDriver(SeleniumDriverCore):
                     'api_key': self.api_key,
                     'label': element_name,
                     'test_case_name': self.test_case_uuid}
-            classify_url = self.url + '/detect'
-            r = requests.post(classify_url, json=data, verify=False).json()
+            r = self.make_json_post_request('/detect', data, self.detect_timeout_message, self.detect_timeout)
             if r['success']:
                 return r
             self._scroll_page(window_height)
@@ -324,6 +316,15 @@ class SmartDriver(SeleniumDriverCore):
                         best_response = r
             return best_response
 
+    def _match_bounding_box_to_selenium_element_js(self, bounding_box, multiplier=1):
+        multiplier = max(multiplier, 1)
+        new_box = {'x': bounding_box['x'] / multiplier, 'y': bounding_box['y'] / multiplier, 'width': bounding_box['width'] / multiplier, 'height': bounding_box['height'] / multiplier}
+
+        middle = (new_box['x'] + new_box['width'] / 2, new_box['y'] + new_box['height'] / 2)
+        # get the element using "return document.elementFromPoint(arguments[0], arguments[1])"
+
+        element = self.driver.execute_script("return document.elementFromPoint(arguments[0], arguments[1])", middle[0], middle[1])
+        return element
 
 
     def _match_bounding_box_to_selenium_element(self, bounding_box, multiplier=1, offset=0):
@@ -331,6 +332,9 @@ class SmartDriver(SeleniumDriverCore):
             We have to ba hacky about this becasue Selenium does not let us click by coordinates.
             We retrieve all elements, compute the IOU between the bounding_box and all the elements and pick the best match.
         """
+        if self.use_fast_js_chopper:
+            return self._match_bounding_box_to_selenium_element_js(bounding_box, multiplier=multiplier)
+
         # Adapt box to local coordinates
         multiplier = max(1, multiplier) # Make sure multiplier isn't 0.
         new_box = {'x': bounding_box['x'] / multiplier, 'y': bounding_box['y'] / multiplier,
@@ -361,6 +365,7 @@ class SmartDriver(SeleniumDriverCore):
             """
             composite = filter(lambda x: x[0] > 0.2, composite)
             composite = list(filter(lambda x: self._center_hit(new_box, x[1].rect), composite))
+            end = time.time()
 
             if len(composite) == 0:
                 # No elements found matching, continue to next search
@@ -421,6 +426,10 @@ class ai_elem(webdriver.remote.webelement.WebElement):
         self._tag_name = elem.get('class', '')
         self._cx = elem.get('x', 0)/multiplier + elem.get('width', 0) / multiplier / 2
         self._cy = elem.get('y', 0)/multiplier + elem.get('height', 0) /multiplier / 2
+        self._top_left = {'x': elem.get('x', 0)/multiplier, 'y': elem.get('y', 0)/multiplier}
+        self._top_right = {'x': elem.get('x', 0)/multiplier + elem.get('width', 0)/multiplier, 'y': elem.get('y', 0)/multiplier}
+        self._bottom_left = {'x': elem.get('x', 0)/multiplier, 'y': elem.get('y', 0)/multiplier + elem.get('height', 0)/multiplier}
+        self._bottom_right = {'x': elem.get('x', 0)/multiplier + elem.get('width', 0)/multiplier, 'y': elem.get('y', 0)/multiplier + elem.get('height', 0)/multiplier}
 
     @property
     def size(self):
@@ -435,6 +444,25 @@ class ai_elem(webdriver.remote.webelement.WebElement):
     def tag_name(self):
         return self._tag_name
 
+    def drag(self, tx, ty, start_x_offset=0, start_y_offset=0):
+        self.driver.execute_cdp_cmd('Input.dispatchMouseEvent',
+                                    {'type': 'mousePressed', 'button': 'left', 'clickCount': 1, 'x': self._cx + start_x_offset,
+                                     'y': self._cy + start_y_offset})
+        time.sleep(0.07)
+        self.driver.execute_cdp_cmd('Input.dispatchMouseEvent',
+                                    {'type': 'mouseMoved', 'button': 'left', 'x': self._cx + start_x_offset + tx,
+                                     'y': self._cy + start_y_offset + ty})
+        time.sleep(0.15)
+        self.driver.execute_cdp_cmd('Input.dispatchMouseEvent',
+                                    {'type': 'mouseReleased', 'button': 'left', 'x': self._cx + start_x_offset + tx,
+                                     'y': self._cy + start_y_offset + ty})
+
+    def highlight(self):
+        self.drag(self._bottom_right['x'] - self._top_left['x'], self._bottom_right['y'] - self._top_left['y'],
+                  start_x_offset=(self._top_left['x'] - self._bottom_right['x']) / 2,
+                  start_y_offset=(self._top_left['y'] - self._bottom_right['y']) / 2)
+
+
     def click(self, js_click=False, use_cdp=False):
         if self._is_real_elem == True and use_cdp == False:
             if not js_click:
