@@ -60,6 +60,11 @@ class SeleniumDriverCore(object):
         self.train = initialization_options.get('train', False)
         self.use_cdp = initialization_options.get('use_cdp', False)
         self.use_ai_elem = initialization_options.get('use_ai_elem', False)
+        self.use_fast_js_chopper = initialization_options.get('use_fast_js_chopper', False)
+        self.detect_timeout = initialization_options.get('detect_timeout', 60)
+        self.misc_timeout = initialization_options.get('misc_timeout', 10)
+        self.do_exact_match_first = initialization_options.get('do_exact_match_first', False)
+
         self.url = initialization_options.get('server_url',
                 os.environ.get('DEVTOOLSAI_URL', 'https://smartdriver.dev-tools.ai'))
         self.default_prod_url = 'https://smartdriver.dev-tools.ai'
@@ -93,6 +98,9 @@ class SeleniumDriverCore(object):
         requests.packages.urllib3.disable_warnings()
         warnings.filterwarnings("ignore", category=DeprecationWarning)
 
+        self.misc_timeout_message =  'devtools_ai service timeout, probably under heavy load or slow connection, doubling the misc_timeout, you can also manually set the misc_timeout to a higher value in initialization_dict, current value: %s s'
+        self.detect_timeout_message =  'devtools_ai service timeout, probably under heavy load or slow connection, doubling the detect_timeout, you can also manually set the detect_timeout to a higher value in initialization_dict, current value: %s s'
+
         for a_name in dir(self.driver):
             if a_name in dir(self):
                 continue
@@ -410,6 +418,7 @@ class SeleniumDriverCore(object):
                         # Key is None when element is frozen or another issue during screenshot and upload.
                         self._update_elem(driver_element, key, element_name)
                 except Exception as e:
+                    log.exception(e)
                     log.error('error uploading screenshot to Dev Tools. Continuing.')
             return driver_element
         except NoElementFoundException as e:
@@ -442,6 +451,50 @@ class SeleniumDriverCore(object):
         except Exception:
             pass
 
+    def make_json_post_request(self, route, data, timeout_error_message, timeout_variable, generic_error_message=None, tries=3):
+        # Verify is False as the lets encrypt certificate raises issue on mac.
+        res = {'success': False, 'message': 'did not run'}
+        local_timeout = timeout_variable
+        if generic_error_message is None:
+            generic_error_message = 'Error making request to ' + route
+        for _ in range(tries):
+            try:
+                url = self.url.rstrip('/') + route
+                res = requests.post(url, json=data, verify=False, timeout=local_timeout).json()
+                break
+            except requests.exceptions.ConnectTimeout:
+                local_timeout = local_timeout * 2
+                log.debug(route)
+                try: # just in case the timeout message does not have a variable to format to too many variablas to format
+                    log.error(timeout_error_message % local_timeout)
+                    log.error(generic_error_message)
+                except Exception:
+                    log.error(timeout_error_message)
+            except requests.exceptions.ConnectionError as e:
+                if 'Connection aborted' in str(e) or 'timeout' in str(e):
+                    local_timeout = local_timeout * 2
+                    log.debug(route)
+                    try:  # just in case the timeout message does not have a variable to format to too many variablas to format
+                        log.error(timeout_error_message % local_timeout)
+                        log.error(generic_error_message)
+                    except Exception:
+                        log.error(timeout_error_message)
+                else:
+                    raise e
+            except requests.exceptions.ReadTimeout as e:
+                local_timeout = local_timeout * 2
+                log.debug(route)
+                try:  # just in case the timeout message does not have a variable to format to too many variablas to format
+                    log.error(timeout_error_message % local_timeout)
+                except Exception:
+                    log.error(timeout_error_message)
+                    log.error(generic_error_message)
+            except Exception as e:
+                log.info(f'Exception type: {type(e)} and {e}')
+                log.exception(e)
+                log.error(generic_error_message)
+        return res
+
     def _update_elem(self, elem, screenshot_uuid, element_name, train_if_necessary=True):
         data = {
             'screenshot_uuid': screenshot_uuid,
@@ -458,15 +511,7 @@ class SeleniumDriverCore(object):
             'ref_screenshot_uuid': self.ref_screenshot_uuid
         }
         try:
-            action_url = self.url + '/add_action_info'
-            # Verify is False as the lets encrypt certificate raises issue on mac.
-            for _ in range(2):
-                try:
-                    res = requests.post(action_url, json=data, verify=False, timeout=4)
-                    res = res.json()
-                    break
-                except Exception:
-                    log.error('error loading add_action_info')
+            res = self.make_json_post_request('/add_action_info', data, self.misc_timeout_message, self.misc_timeout)
             if not res['success']:
                 log.error(res['message'])
         except Exception:
@@ -489,14 +534,8 @@ class SeleniumDriverCore(object):
 
     def _check_screenshot_exists(self, screenshot_uuid, element_name):
         data = {'api_key': self.api_key, 'screenshot_uuid': screenshot_uuid, 'label': element_name}
-        check_screenshot_url = self.url + '/exists_screenshot'
         start = time.time()
-        for _ in range(2):
-            try:
-                r = requests.post(check_screenshot_url, json=data, verify=False, timeout=6).json()
-                break
-            except Exception:
-                log.error('error in exists_screenshot')
+        r = self.make_json_post_request('/exists_screenshot', data, self.misc_timeout_message, self.misc_timeout)
         end = time.time()
         if self.debug:
             print(f'Cached bounding box request time: {end - start}')
@@ -506,16 +545,10 @@ class SeleniumDriverCore(object):
         return r
 
     def _check_frozen(self, element_name):
-        url = self.url + '/check_frozen'
         request_data = {'api_key': self.api_key,
                         'label': element_name
                         }
-        for _ in range(2):
-            try:
-                resp = requests.post(url, json=request_data, timeout=3).json()
-                break
-            except Exception:
-                log.error('error in check_frozen')
+        resp = self.make_json_post_request('/check_frozen', request_data, self.misc_timeout_message, self.misc_timeout)
         return resp.get('is_frozen', True)
 
     def _scroll_page(self, amount):
@@ -570,14 +603,8 @@ class SeleniumDriverCore(object):
                 if self._driver_type == 'appium':
                     data['is_appium'] = True
                     del data['screenshot_uuid']
-                upload_screenshot_url = self.url + '/upload_screenshot'
                 start = time.time()
-                for _ in range(2):
-                    try:
-                        r = requests.post(upload_screenshot_url, json=data, verify=False, timeout=6).json()
-                        break
-                    except Exception:
-                        log.error('error uploading url')
+                r = self.make_json_post_request('/upload_screenshot', data, self.misc_timeout_message, self.misc_timeout)
                 end = time.time()
                 if r['success'] == False:
                     log.error(f'Error uploading screenshot {screenshot_uuid}')
@@ -601,15 +628,8 @@ class SeleniumDriverCore(object):
                 'event_id': event_id}
 
         needs_reload = False
-        r = {'success': False}
+        r = self.make_json_post_request('/testcase/get_action_info', data, self.misc_timeout_message, max(15, self.misc_timeout))
         has_training_data = False
-        for _ in range(2):
-            try:
-                r = requests.post(self.url + '/testcase/get_action_info', json=data, verify=False, timeout=15).json()
-                break
-            except Exception as e:
-                log.error(e)
-                log.error('error getting test case actioninfo')
         if r['success']:
             box = r['predicted_element']
             needs_reload = r['needs_reload']
@@ -623,7 +643,6 @@ class SeleniumDriverCore(object):
         """
             Uploads the screenshot to the server for test creation and retrieves the uuid / hash / key in return.
         """
-        url = self.url + '/upload_screenshot'
         screenshotBase64 = self._get_screenshot()
         self.last_screenshot = screenshotBase64
         data = {'api_key': self.api_key,
@@ -633,12 +652,7 @@ class SeleniumDriverCore(object):
                 'is_interactive': True}
         if self._driver_type == 'appium':
             data['is_appium'] = True
-        for _ in range(2):
-            try:
-                res = requests.post(url, json=data, verify=False, timeout=6).json()
-                break
-            except Exception:
-                log.error('error uploading screenshot')
+        res = self.make_json_post_request('/upload_screenshot', data, self.misc_timeout_message, self.misc_timeout)
         if res['success']:
             self.last_test_case_screenshot_uuid = res['screenshot_uuid']
             self.last_screenshot = screenshotBase64
@@ -683,7 +697,7 @@ class SeleniumDriverCore(object):
         data = {'real_xpath': real_xpath, 'label': label, 'screenshot_uuid': screenshot_uuid, 'api_key': self.api_key,
                 'warning_type': warning_type,
                 'predicted_element': element_box, 'score': score, 'message': message}
-        r = requests.post(self.url + '/save_warning', json=data, verify=False)
+        r = self.make_json_post_request('/save_warning', data, self.misc_timeout_message, self.misc_timeout)
 
     def _generate_xpath(self, element):
         return self.driver.execute_script(
@@ -743,4 +757,4 @@ class SeleniumDriverCore(object):
             "} return absoluteXPath(arguments[0]);", element)
 
 class NoElementFoundException(Exception):
-    pass
\ No newline at end of file
+    pass
